package com.sun.org.apache.xml.internal.security.signature;

import com.sun.org.apache.xml.internal.security.transforms.params.InclusiveNamespaces;
import javax.crypto.spec.SecretKeySpec;
import javax.crypto.SecretKey;
import java.io.OutputStream;
import com.sun.org.apache.xml.internal.security.c14n.InvalidCanonicalizerException;
import com.sun.org.apache.xml.internal.security.c14n.CanonicalizationException;
import org.w3c.dom.Node;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.SAXException;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import java.io.ByteArrayInputStream;
import com.sun.org.apache.xml.internal.security.c14n.Canonicalizer;
import com.sun.org.apache.xml.internal.security.utils.XMLUtils;
import com.sun.org.apache.xml.internal.security.exceptions.XMLSecurityException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import com.sun.org.apache.xml.internal.security.algorithms.SignatureAlgorithm;

public class SignedInfo extends Manifest
{
    private SignatureAlgorithm signatureAlgorithm;
    private byte[] c14nizedBytes;
    private Element c14nMethod;
    private Element signatureMethod;
    
    public SignedInfo(Document document) throws XMLSecurityException {
        this(document, "http://www.w3.org/2000/09/xmldsig#dsa-sha1", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
    }
    
    public SignedInfo(Document document, String s, String s2) throws XMLSecurityException {
        this(document, s, 0, s2);
    }
    
    public SignedInfo(Document document, String s, int hmacOutputLength, String s2) throws XMLSecurityException {
        super(document);
        (this.c14nMethod = XMLUtils.createElementInSignatureSpace(this.getDocument(), "CanonicalizationMethod")).setAttributeNS(null, "Algorithm", s2);
        this.appendSelf(this.c14nMethod);
        this.addReturnToSelf();
        if (hmacOutputLength > 0) {
            this.signatureAlgorithm = new SignatureAlgorithm(this.getDocument(), s, hmacOutputLength);
        }
        else {
            this.signatureAlgorithm = new SignatureAlgorithm(this.getDocument(), s);
        }
        this.appendSelf(this.signatureMethod = this.signatureAlgorithm.getElement());
        this.addReturnToSelf();
    }
    
    public SignedInfo(Document document, Element element, Element c14nMethod) throws XMLSecurityException {
        super(document);
        this.appendSelf(this.c14nMethod = c14nMethod);
        this.addReturnToSelf();
        this.signatureAlgorithm = new SignatureAlgorithm(element, null);
        this.appendSelf(this.signatureMethod = this.signatureAlgorithm.getElement());
        this.addReturnToSelf();
    }
    
    public SignedInfo(Element element, String s) throws XMLSecurityException {
        this(element, s, true);
    }
    
    public SignedInfo(Element element, String s, boolean secureValidation) throws XMLSecurityException {
        super(reparseSignedInfoElem(element, secureValidation), s, secureValidation);
        this.c14nMethod = XMLUtils.getNextElement(element.getFirstChild());
        this.signatureMethod = XMLUtils.getNextElement(this.c14nMethod.getNextSibling());
        this.signatureAlgorithm = new SignatureAlgorithm(this.signatureMethod, this.getBaseURI(), secureValidation);
    }
    
    private static Element reparseSignedInfoElem(Element element, boolean secureValidation) throws XMLSecurityException {
        String attributeNS = XMLUtils.getNextElement(element.getFirstChild()).getAttributeNS(null, "Algorithm");
        if (!attributeNS.equals("http://www.w3.org/TR/2001/REC-xml-c14n-20010315") && !attributeNS.equals("http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments") && !attributeNS.equals("http://www.w3.org/2001/10/xml-exc-c14n#") && !attributeNS.equals("http://www.w3.org/2001/10/xml-exc-c14n#WithComments") && !attributeNS.equals("http://www.w3.org/2006/12/xml-c14n11") && !attributeNS.equals("http://www.w3.org/2006/12/xml-c14n11#WithComments")) {
            try {
                Canonicalizer instance = Canonicalizer.getInstance(attributeNS);
                instance.setSecureValidation(secureValidation);
                byte[] canonicalizeSubtree = instance.canonicalizeSubtree(element);
                DocumentBuilder documentBuilder = XMLUtils.createDocumentBuilder(false, secureValidation);
                try (final ByteArrayInputStream is = new ByteArrayInputStream(canonicalizeSubtree)) {
                    Node importNode = element.getOwnerDocument().importNode(documentBuilder.parse(is).getDocumentElement(), true);
                    element.getParentNode().replaceChild(importNode, element);
                    return (Element)importNode;
                }
            }
            catch (ParserConfigurationException originalException) {
                throw new XMLSecurityException(originalException);
            }
            catch (IOException originalException2) {
                throw new XMLSecurityException(originalException2);
            }
            catch (SAXException originalException3) {
                throw new XMLSecurityException(originalException3);
            }
            return element;
        }
        return element;
    }
    
    public boolean verify() throws MissingResourceFailureException, XMLSecurityException {
        return super.verifyReferences(false);
    }
    
    public boolean verify(boolean b) throws MissingResourceFailureException, XMLSecurityException {
        return super.verifyReferences(b);
    }
    
    public byte[] getCanonicalizedOctetStream() throws CanonicalizationException, InvalidCanonicalizerException, XMLSecurityException {
        if (this.c14nizedBytes == null) {
            Canonicalizer instance = Canonicalizer.getInstance(this.getCanonicalizationMethodURI());
            instance.setSecureValidation(this.isSecureValidation());
            String inclusiveNamespaces = this.getInclusiveNamespaces();
            if (inclusiveNamespaces == null) {
                this.c14nizedBytes = instance.canonicalizeSubtree(this.getElement());
            }
            else {
                this.c14nizedBytes = instance.canonicalizeSubtree(this.getElement(), inclusiveNamespaces);
            }
        }
        return (byte[])this.c14nizedBytes.clone();
    }
    
    public void signInOctetStream(OutputStream writer) throws CanonicalizationException, InvalidCanonicalizerException, XMLSecurityException {
        if (this.c14nizedBytes == null) {
            Canonicalizer instance = Canonicalizer.getInstance(this.getCanonicalizationMethodURI());
            instance.setSecureValidation(this.isSecureValidation());
            instance.setWriter(writer);
            String inclusiveNamespaces = this.getInclusiveNamespaces();
            if (inclusiveNamespaces == null) {
                instance.canonicalizeSubtree(this.getElement());
            }
            else {
                instance.canonicalizeSubtree(this.getElement(), inclusiveNamespaces);
            }
        }
        else {
            try {
                writer.write(this.c14nizedBytes);
            }
            catch (IOException cause) {
                throw new RuntimeException(cause);
            }
        }
    }
    
    public String getCanonicalizationMethodURI() {
        return this.c14nMethod.getAttributeNS(null, "Algorithm");
    }
    
    public String getSignatureMethodURI() {
        Element signatureMethodElement = this.getSignatureMethodElement();
        if (signatureMethodElement != null) {
            return signatureMethodElement.getAttributeNS(null, "Algorithm");
        }
        return null;
    }
    
    public Element getSignatureMethodElement() {
        return this.signatureMethod;
    }
    
    public SecretKey createSecretKey(byte[] key) {
        return new SecretKeySpec(key, this.signatureAlgorithm.getJCEAlgorithmString());
    }
    
    public SignatureAlgorithm getSignatureAlgorithm() {
        return this.signatureAlgorithm;
    }
    
    @Override
    public String getBaseLocalName() {
        return "SignedInfo";
    }
    
    public String getInclusiveNamespaces() {
        String canonicalizationMethodURI = this.getCanonicalizationMethodURI();
        if (!canonicalizationMethodURI.equals("http://www.w3.org/2001/10/xml-exc-c14n#") && !canonicalizationMethodURI.equals("http://www.w3.org/2001/10/xml-exc-c14n#WithComments")) {
            return null;
        }
        Element nextElement = XMLUtils.getNextElement(this.c14nMethod.getFirstChild());
        if (nextElement != null) {
            try {
                return new InclusiveNamespaces(nextElement, "http://www.w3.org/2001/10/xml-exc-c14n#").getInclusiveNamespaces();
            }
            catch (XMLSecurityException ex) {
                return null;
            }
        }
        else {
            return null;
        }
    }
}
